function[x0,penalty_value]=maxL1NormLagrangian(A,B,Q,P,N,Xc, u_min, u_max,M)
%% Define system values
n = size(A,1); %number of states
m = size(B,2); %number of inputs

[H,F,G,W,E] = matrixCompactForm2011(A,B,Q,P,N, Xc, u_min, u_max); %Recast MPC problem to compact form. Obj: J=u^(T)Hu+x_0^(T)Fu. Constraints: G*u <= W+E*x_0
%% Calculate decision-dimensions

%Form of z: z=[x0,u,lambda,s,delta,muh]

nc = length(G); % Number of constraints = number of lambdas = number of s
z_length = n+m*N+2*nc+2*nc+N*m; %Number of optimization variables.

%% Generate inequality constraint matrix, Aleq*z<=bleq
Aleq = [-E G zeros(nc,z_length-n-m*N);
        E -G zeros(nc,nc) M*eye(nc) zeros(nc,z_length-n-m*N-nc-nc);
        zeros(nc,n) zeros(nc,m*N) -eye(nc) zeros(nc,z_length-n-m*N-nc);
        zeros(nc,n) zeros(nc,m*N) eye(nc) -M*eye(nc) zeros(nc,z_length-n-m*N-nc-nc);
        zeros(2*nc,n) zeros(2*nc,m*N) zeros(2*nc,2*nc) -eye(2*nc) zeros(2*nc,z_length-n-m*N-2*nc-2*nc);
        zeros(nc,n) zeros(nc,m*N) zeros(nc,nc) M*eye(nc) eye(nc) zeros(nc,z_length-n-m*N-3*nc);
        zeros(nc,n) zeros(nc,m*N) zeros(nc,nc) M*eye(nc) zeros(nc,nc) eye(nc) zeros(nc,z_length-n-m*N-nc-nc-2*nc)];
        
bleq = [W;
        ones(nc,1)*M-W;
        zeros(nc,1);
        zeros(nc,1);
        zeros(2*nc,1);
        M*ones(2*nc,1)];
    

%% Generate equality constraint matrix, Aeq*z==beq

Aeq = [zeros(size(G',1),size(F,2)), zeros(size(G',1),size(H,2)) G' zeros(size(G',1),z_length-m*N-nc-n);
       zeros(nc,n+N*m) eye(nc) zeros(nc,nc) -eye(nc) eye(nc) G];
       
beq =  [-H';
        zeros(nc,1)];

%% Define and solve optimization
ctype = [char(ones(n+m*N+nc,1)*char('C'));char(ones(nc,1)*char('B'));char(ones(2*nc+N*m,1)*char('C'))]'; % Define continuous and binary varuables. Only s binary
f = [zeros(1, n+m*N+m*N*2) -ones(1,nc-(m*N*2)) zeros(1,nc+2*nc+N*m)];%Objective function max||lambda||_1
[z,fval,exitflag,output] = cplexmilp(f,Aleq,bleq,Aeq,beq,[],[],[],[],[],ctype); %Solve optimization problem

%% Check variables (troubleshooting)
lambda_x = z(n+m*N+m*N*2+1:n+m*N+nc,1);
x0 = z(1:n,1);


%% Penalty value
penalty_value = sum(lambda_x); % =||lambda||_1