function[H,F,G,W,E,c]=matrixCompactForm(A,B,Q,P,N, Xc, u_min, u_max)
%%   function[H,F,G,W,E,c]=matrixCompactForm(A,B,Q,P,N, Xc, u_min, u_max)
%
%    Function recasts the MPC problem to compact form
%
%    A: is the system matrix
%    B: is the input matrix
%    Q: is the cost function weight on x
%    P: is the cost function weight on u
%    N: is the horizon
%    Xc: is the state constraints in augmented form
%    u_min and u_max: are the min and max values of u, respectively

%%
n = size(A,1); %number of states
m = size(B,2); %number of inputs

%% Generate A_hat
for i = 1:N
   A_hat(1+n*(i-1):1+n*(i-1)+(n-1),1:n) = A^(i);
end

%% Generate B_hat
for i=1:N
    for j = 1:N
         if j-i<0
            B_hat(1+n*(j-1):1+n*(j-1)+(n-1),1+m*(i-1):1+m*(i-1)+(m-1)) = zeros(n,m);
         else
            B_hat(1+n*(j-1):1+n*(j-1)+(n-1),1+m*(i-1):1+m*(i-1)+(m-1)) = A^(j-1-(i-1))*B; 
         end     
    end
end

%% Design state constraint matrices
I_hat = kron(eye(N),Xc(:,1:n));
J_hat = repmat(Xc(:,1+n),N,1);

K = I_hat*A_hat;
L = I_hat*B_hat;


%% Generate Q_hat and P_hat
Q_hat = repmat(Q,1,N);
P_hat = repmat(P,1,N);

%% Generate H and F. Matrices that are used in J=-Qx+Pu=Hu-Fx0 linear cost function.
H = P_hat-Q_hat*B_hat;
F = Q_hat*A_hat;  

%% Generate constraint matrices (G,W,E) used in compact notation for model constraints.  G*u <= W+E*x_0
% 
c = [zeros(2*m*N,1); ones(length(L),1)];
G = [ eye(m*N);
     -eye(m*N);
      L];
 
 W = [ repmat(u_max, N, 1);
       repmat(-u_min, N, 1);
       J_hat];

 E = [ zeros(2*N*m,n);
       -K];


 [G,W,E,c] = SortLowerTriangular(G,W,E,c);
