function[penalty_value]=maxLinfNormLagrangian(A,B,Q,P,N,Xc,u_min,u_max,M)
%%   Usage: function[penalty_value]=maxLinfNormLagrangian(A,B,Q,P,N,Xc,u_min,u_max,M) 
%
%    Function calculates max||lambda||_1 for the MPC problem with linear cost where
%
%    A: is the system matrix
%    B: is the input matrix
%    Q: is the cost function weight on x
%    P: is the cost function weight on u
%    N: is the horizon
%    Xc: is the state constraints in augmented form
%    u_min and u_max: are the min and max values of u, respectively
%    M: is the big-M value to be used in the MILP

%% Define system values
n = size(A,1); %number of states
m = size(B,2); %number of inputs

[H,F,G,W,E,c] = matrixCompactForm(A,B,Q,P,N,Xc,u_min,u_max); %Recast MPC problem to compact form. Obj: J=u^(T)Hu+x_0^(T)Fu. Constraints: G*u <= W+E*x_0

%% Calculate decision-dimensions

%Form of z: z=[x0,u,lambda,s]

nc = length(G); % Number of constraints = number of lambdas = number of s
z_length = n+m*N+2*nc; %Number of optimization variables.


%% Define LICQ constraints
   
qk = size(Xc(:,1:n),1)+2*m; 
q = size(G,1);

for k=1:N 
    startvar = (k-1)*qk;
    
    tg1(k,:) = [zeros(1,startvar) ones(1,qk) zeros(1,q-qk-startvar)];
    tw1(k,1) = min(qk,(k)*m);
    
end

for k=1:N
    tau = (k)*qk;
    
    tg2(k,:) = [ones(1,tau) zeros(1,q-tau)];
    tw2(k,1) = (k)*m;
    
end

tg=[tg1;tg2];
tw=[tw1;tw2];


%% Generate inequality constraint matrix, Aleq*z<=bleq
Aleq = [-E G zeros(nc,z_length-n-m*N);
        E -G zeros(nc,nc) M*eye(nc) zeros(nc,z_length-n-m*N-nc-nc);
        zeros(nc,n) zeros(nc,m*N) -eye(nc) zeros(nc,z_length-n-m*N-nc);
        zeros(nc,n) zeros(nc,m*N) eye(nc) -M*eye(nc) zeros(nc,z_length-n-m*N-nc-nc);
        zeros(size(tg,1),n+m*N+nc) tg zeros(size(tg,1),z_length-n-m*N-nc-size(tg,2))];
        
bleq = [W;
        ones(nc,1)*M-W;
        zeros(nc,1);
        zeros(nc,1);
        tw];

    

%% Generate equality constraint matrix, Aeq*z==beq

Aeq = [zeros(size(G',1),size(F,2)), zeros(size(G',1),size(H,2)) G' zeros(size(G',1),z_length-m*N-nc-n)];
       
beq = -H';

%% Define and solve optimization
ctype = [char(ones(n+m*N+nc,1)*char('C'));char(ones(nc,1)*char('B'))]'; % Define continuous and binary varuables. Only s binary
penalty_value = 0;

counter = 1;
for j=1:nc
    disp(['Iteration',' ', num2str(j),':',num2str(nc)])
    if c(j)==1
        f = [zeros(1, n+m*N+j-1) -1 zeros(1,z_length-(n+m*N+j))]; %Objective function max lambda
        [z,fval,exitflag,output] = cplexmilp(f,Aleq,bleq,Aeq,beq,[],[],[],[],[],ctype); %Solve optimization problem
        lambda_temp = z(n+m*N+j)
        lambdas(counter,1) = lambda_temp;
    else 
        disp('Constraint not in relevant subset. Skipping...')
    end
counter = counter + 1;
end



%% Penalty value
lambdas
penalty_value = max(lambdas); % =||lambda||_max
